@@ -1,19 +1,10 @@
 // Copyright (C) 2020 Mathieu Duponchelle <mathieu@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::glib;
 use gst::prelude::*;
@@ -45,6 +36,7 @@ const DEFAULT_BLACK_BACKGROUND: bool = false;
 struct Settings {
     field: i32,
     black_background: bool,
+    timeout: Option<gst::ClockTime>,
 }
 
 impl Default for Settings {
@@ -52,6 +44,7 @@ impl Default for Settings {
         Settings {
             field: DEFAULT_FIELD,
             black_background: DEFAULT_BLACK_BACKGROUND,
+            timeout: gst::ClockTime::NONE,
         }
     }
 }
@@ -64,6 +57,7 @@ struct State {
     left_alignment: i32,
     attach: bool,
     selected_field: Option<u8>,
+    last_cc_pts: Option<gst::ClockTime>,
 }
 
 impl Default for State {
@@ -76,6 +70,7 @@ impl Default for State {
             left_alignment: 0,
             attach: false,
             selected_field: None,
+            last_cc_pts: gst::ClockTime::NONE,
         }
     }
 }
@@ -142,20 +137,20 @@ impl Cea608Overlay {
                 "12345678901234567890123456789012\n2\n3\n4\n5\n6\n7\n8\n9\n0\n1\n2\n3\n4\n5",
             );
             let (_ink_rect, logical_rect) = layout.extents();
-            if logical_rect.width > video_info.width() as i32 * pango::SCALE
-                || logical_rect.height > video_info.height() as i32 * pango::SCALE
+            if logical_rect.width() > video_info.width() as i32 * pango::SCALE
+                || logical_rect.height() > video_info.height() as i32 * pango::SCALE
             {
                 font_desc.set_size((font_size - 1) * pango::SCALE);
                 layout.set_font_description(Some(&font_desc));
                 break;
             }
-            left_alignment = (video_info.width() as i32 - logical_rect.width / pango::SCALE) / 2;
+            left_alignment = (video_info.width() as i32 - logical_rect.width() / pango::SCALE) / 2;
             font_size += 1;
         }
 
         if self.settings.lock().unwrap().black_background {
             let attrs = pango::AttrList::new();
-            let attr = pango::Attribute::new_background(0, 0, 0);
+            let attr = pango::AttrColor::new_background(0, 0, 0);
             attrs.insert(attr);
             layout.set_attributes(Some(&attrs));
         }
@@ -171,8 +166,8 @@ impl Cea608Overlay {
         let layout = state.layout.as_ref().unwrap();
         layout.set_text(text);
         let (_ink_rect, logical_rect) = layout.extents();
-        let height = logical_rect.height / pango::SCALE;
-        let width = logical_rect.width / pango::SCALE;
+        let height = logical_rect.height() / pango::SCALE;
+        let width = logical_rect.width() / pango::SCALE;
 
         // No text actually needs rendering
         if width == 0 || height == 0 {
@@ -331,6 +326,7 @@ impl Cea608Overlay {
         element: &super::Cea608Overlay,
         state: &mut State,
         data: &[u8],
+        pts: gst::ClockTime,
     ) {
         if data.len() % 3 != 0 {
             gst_warning!(CAT, "cc_data length is not a multiple of 3, truncating");
@@ -353,27 +349,26 @@ impl Cea608Overlay {
                     }
 
                     if Some(cc_type) == state.selected_field {
-                        match state
+                        if let Ok(Status::Ready) = state
                             .caption_frame
                             .decode((triple[1] as u16) << 8 | triple[2] as u16, 0.0)
                         {
-                            Ok(Status::Ready) => {
-                                let text = match state.caption_frame.to_text(true) {
-                                    Ok(text) => text,
-                                    Err(_) => {
-                                        gst_error!(
-                                            CAT,
-                                            obj: pad,
-                                            "Failed to convert caption frame to text"
-                                        );
-                                        continue;
-                                    }
-                                };
+                            let text = match state.caption_frame.to_text(true) {
+                                Ok(text) => text,
+                                Err(_) => {
+                                    gst_error!(
+                                        CAT,
+                                        obj: pad,
+                                        "Failed to convert caption frame to text"
+                                    );
+                                    continue;
+                                }
+                            };
 
-                                self.overlay_text(element, &text, state);
-                            }
-                            _ => (),
+                            self.overlay_text(element, &text, state);
                         }
+
+                        self.reset_timeout(state, pts);
                     }
                 } else {
                     break;
@@ -388,6 +383,7 @@ impl Cea608Overlay {
         element: &super::Cea608Overlay,
         state: &mut State,
         data: &[u8],
+        pts: gst::ClockTime,
     ) {
         if data.len() % 3 != 0 {
             gst_warning!(CAT, "cc_data length is not a multiple of 3, truncating");
@@ -406,31 +402,30 @@ impl Cea608Overlay {
             }
 
             if Some(cc_type) == state.selected_field {
-                match state
+                if let Ok(Status::Ready) = state
                     .caption_frame
                     .decode((triple[1] as u16) << 8 | triple[2] as u16, 0.0)
                 {
-                    Ok(Status::Ready) => {
-                        let text = match state.caption_frame.to_text(true) {
-                            Ok(text) => text,
-                            Err(_) => {
-                                gst_error!(
-                                    CAT,
-                                    obj: pad,
-                                    "Failed to convert caption frame to text"
-                                );
-                                continue;
-                            }
-                        };
+                    let text = match state.caption_frame.to_text(true) {
+                        Ok(text) => text,
+                        Err(_) => {
+                            gst_error!(CAT, obj: pad, "Failed to convert caption frame to text");
+                            continue;
+                        }
+                    };
 
-                        self.overlay_text(element, &text, state);
-                    }
-                    _ => (),
+                    self.overlay_text(element, &text, state);
                 }
+
+                self.reset_timeout(state, pts);
             }
         }
     }
 
+    fn reset_timeout(&self, state: &mut State, pts: gst::ClockTime) {
+        state.last_cc_pts = Some(pts);
+    }
+
     fn sink_chain(
         &self,
         pad: &gst::Pad,
@@ -439,6 +434,11 @@ impl Cea608Overlay {
     ) -> Result<gst::FlowSuccess, gst::FlowError> {
         gst_log!(CAT, obj: pad, "Handling buffer {:?}", buffer);
 
+        let pts = buffer.pts().ok_or_else(|| {
+            gst_error!(CAT, obj: pad, "Require timestamped buffers");
+            gst::FlowError::Error
+        })?;
+
         let mut state = self.state.lock().unwrap();
 
         if self.srcpad.check_reconfigure() {
@@ -453,7 +453,7 @@ impl Cea608Overlay {
             if meta.caption_type() == gst_video::VideoCaptionType::Cea708Cdp {
                 match extract_cdp(meta.data()) {
                     Ok(data) => {
-                        self.decode_cc_data(pad, element, &mut state, data);
+                        self.decode_cc_data(pad, element, &mut state, data, pts);
                     }
                     Err(e) => {
                         gst_warning!(CAT, "{}", &e.to_string());
@@ -461,34 +461,43 @@ impl Cea608Overlay {
                     }
                 }
             } else if meta.caption_type() == gst_video::VideoCaptionType::Cea708Raw {
-                self.decode_cc_data(pad, element, &mut state, meta.data());
+                self.decode_cc_data(pad, element, &mut state, meta.data(), pts);
             } else if meta.caption_type() == gst_video::VideoCaptionType::Cea608S3341a {
-                self.decode_s334_1a(pad, element, &mut state, meta.data());
+                self.decode_s334_1a(pad, element, &mut state, meta.data(), pts);
             } else if meta.caption_type() == gst_video::VideoCaptionType::Cea608Raw {
                 let data = meta.data();
                 assert!(data.len() % 2 == 0);
                 for i in 0..data.len() / 2 {
-                    match state
+                    if let Ok(Status::Ready) = state
                         .caption_frame
                         .decode((data[i * 2] as u16) << 8 | data[i * 2 + 1] as u16, 0.0)
                     {
-                        Ok(Status::Ready) => {
-                            let text = match state.caption_frame.to_text(true) {
-                                Ok(text) => text,
-                                Err(_) => {
-                                    gst_error!(
-                                        CAT,
-                                        obj: pad,
-                                        "Failed to convert caption frame to text"
-                                    );
-                                    continue;
-                                }
-                            };
+                        let text = match state.caption_frame.to_text(true) {
+                            Ok(text) => text,
+                            Err(_) => {
+                                gst_error!(
+                                    CAT,
+                                    obj: pad,
+                                    "Failed to convert caption frame to text"
+                                );
+                                continue;
+                            }
+                        };
 
-                            self.overlay_text(element, &text, &mut state);
-                        }
-                        _ => (),
+                        self.overlay_text(element, &text, &mut state);
                     }
+
+                    self.reset_timeout(&mut state, pts);
+                }
+            }
+        }
+
+        if let Some(timeout) = self.settings.lock().unwrap().timeout {
+            if let Some(interval) = pts.opt_saturating_sub(state.last_cc_pts) {
+                if interval > timeout {
+                    gst_info!(CAT, obj: element, "Reached timeout, clearing overlay");
+                    state.composition.take();
+                    state.last_cc_pts.take();
                 }
             }
         }
@@ -503,11 +512,9 @@ impl Cea608Overlay {
                     state.video_info.as_ref().unwrap(),
                 )
                 .unwrap();
-                match composition.blend(&mut frame) {
-                    Err(_) => {
-                        gst_error!(CAT, obj: pad, "Failed to blend composition");
-                    }
-                    _ => (),
+
+                if composition.blend(&mut frame).is_err() {
+                    gst_error!(CAT, obj: pad, "Failed to blend composition");
                 }
             }
         }
@@ -573,11 +580,13 @@ impl ObjectSubclass for Cea608Overlay {
                 )
             })
             .flags(gst::PadFlags::PROXY_CAPS)
+            .flags(gst::PadFlags::PROXY_ALLOCATION)
             .build();
 
         let templ = klass.pad_template("src").unwrap();
         let srcpad = gst::Pad::builder_with_template(&templ, Some("src"))
             .flags(gst::PadFlags::PROXY_CAPS)
+            .flags(gst::PadFlags::PROXY_ALLOCATION)
             .build();
 
         Self {
@@ -593,7 +602,7 @@ impl ObjectImpl for Cea608Overlay {
     fn properties() -> &'static [glib::ParamSpec] {
         static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
             vec![
-                glib::ParamSpec::new_int(
+                glib::ParamSpecInt::new(
                     "field",
                     "Field",
                     "The field to render the caption for when available, (-1=automatic)",
@@ -602,13 +611,22 @@ impl ObjectImpl for Cea608Overlay {
                     DEFAULT_FIELD,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_PLAYING,
                 ),
-                glib::ParamSpec::new_boolean(
+                glib::ParamSpecBoolean::new(
                     "black-background",
                     "Black background",
                     "Whether a black background should be drawn behind text",
                     DEFAULT_BLACK_BACKGROUND,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_PLAYING,
                 ),
+                glib::ParamSpecUInt64::new(
+                    "timeout",
+                    "Timeout",
+                    "Duration after which to erase overlay when no cc data has arrived for the selected field",
+                    gst::ClockTime::from_seconds(16).nseconds(),
+                    u64::MAX,
+                    u64::MAX,
+                    glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_PLAYING,
+                ),
             ]
         });
 
@@ -640,6 +658,16 @@ impl ObjectImpl for Cea608Overlay {
                 settings.black_background = value.get().expect("type checked upstream");
                 let _ = state.layout.take();
             }
+            "timeout" => {
+                let mut settings = self.settings.lock().unwrap();
+
+                let timeout = value.get().expect("type checked upstream");
+
+                settings.timeout = match timeout {
+                    u64::MAX => gst::ClockTime::NONE,
+                    _ => Some(gst::ClockTime::from_nseconds(timeout)),
+                };
+            }
             _ => unimplemented!(),
         }
     }
@@ -654,6 +682,14 @@ impl ObjectImpl for Cea608Overlay {
                 let settings = self.settings.lock().unwrap();
                 settings.black_background.to_value()
             }
+            "timeout" => {
+                let settings = self.settings.lock().unwrap();
+                if let Some(timeout) = settings.timeout {
+                    timeout.nseconds().to_value()
+                } else {
+                    u64::MAX.to_value()
+                }
+            }
             _ => unimplemented!(),
         }
     }
@@ -665,6 +701,8 @@ impl ObjectImpl for Cea608Overlay {
     }
 }
 
+impl GstObjectImpl for Cea608Overlay {}
+
 impl ElementImpl for Cea608Overlay {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
