@@ -626,7 +626,7 @@ impl MccParse {
                 }
             };
 
-            let parse = Self::from_instance(&element);
+            let parse = element.imp();
             parse.loop_fn(&element);
         });
         if res.is_err() {
