@@ -514,7 +514,7 @@ impl SccParse {
                 }
             };
 
-            let parse = Self::from_instance(&element);
+            let parse = element.imp();
             parse.loop_fn(&element);
         });
         if res.is_err() {
